<?php
declare(strict_types=1);

namespace GamesShop\Routing\Api\Web;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use GamesShop\Entities\Account\User;
use GamesShop\Login\LoginHandler;
use GamesShop\Login\UserPermission;
use Laminas\Diactoros\Response\EmptyResponse;
use League\Route\Http\Exception\ForbiddenException;
use League\Route\Http\Exception\UnauthorizedException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

final readonly class UserModifyRoute
{
    public function __construct(
        private LoginHandler $loginHandler,
        private EntityManager $entityManager,
    )
    {
    }

    public function __invoke(ServerRequestInterface $request, array $args): ResponseInterface
    {
        if (!$this->loginHandler->isLoggedIn()) {
            throw new UnauthorizedException();
        }

        $user = $this->loginHandler->getCurrentUser();
        if (!$user->getPermission()->hasLevel(UserPermission::ADMIN)) {
            throw new ForbiddenException();
        }

        $permissions = $request->getParsedBody()['permission'];

        $toChangeUser = $this->entityManager->getRepository(User::class)->find((int)$args['id']);
        $toChangeUser->setPermission(UserPermission::from((int)$permissions));

        $this->entityManager->flush();

        return new EmptyResponse(200);
    }
}